searchData={"items":[{"type":"module","title":"FsBuild","doc":"A filesystem-based build engine which provides a flexible mechanism for\nparsing and processing files.\n\n> This is a fork of [NimblePublisher](https://github.com/dashbitco/nimble_publisher).","ref":"FsBuild.html"},{"type":"module","title":"Overview - FsBuild","doc":"```elixir\nuse FsBuild,\n  from: Application.app_dir(:app_name, \"priv/articles/**/*.md\"),\n  adapter: {FsBuild.Adapters.MarkdownPublisher, []},\n  build: Article,\n  as: :articles\n```\n\nThe example above will:\n\n1. get all files whose path match the given pattern.\n2. extract `data` and `metadata` from files by using adapter\n   `FsBuild.Adapters.MarkdownPublisher`.\n3. call `Article.build/3` for each file, passing the `data` and\n   `metadata` arguments.\n4. define a module attribute named `@articles` with all built articles\n   returned by `Article.build/3` function.","ref":"FsBuild.html#module-overview"},{"type":"module","title":"Options - FsBuild","doc":"- `:from` - a wildcard pattern where to find all files.\n- `:adapter` - the adapter and its options - `{module(), any()}`.\n- `:build` - this option can be:\n  - the name of module which has `build/3` function inside.\n  - a function whose arity number is 3.\n- `:as` - the name of the module attribute to store all built files.","ref":"FsBuild.html#module-options"},{"type":"module","title":"An example - FsBuild","doc":"Let's build a blog by using the built-in adapter -\n`FsBuild.Adapters.MarkdownPublisher`, which has Markdown and code\nhighlighting support.\n\nFirst, add `fs_build` and other required packages as dependencies:\n\n    def deps do\n      [\n        {:fs_build, \"~> 1.0\"},\n        {:earmark, \"~> 1.4\"},\n        {:makeup, \"~> 1.0\"},\n        {:makeup_elixir, \">= 0.0.0\"},\n        {:makeup_erlang, \">= 0.0.0\"}\n      ]\n    end\n\nEach post stays in the `priv/posts/` directory with the format:\n\n    priv/posts/YEAR/MONTH-DAY-ID.md\n\nA typical post will look like this:\n\n    # priv/posts/2020/04-17-hello-world.md\n    %{\n      title: \"Hello world!\",\n      author: \"José Valim\",\n      tags: ~w(hello),\n      description: \"Let's learn how to say hello world\"\n    }\n    ---\n    This is the post.\n\nThen, we define a `Post` struct that expects all of the fields\nabove. We will also have a `:date` field that we will build from the\nfilename. Overall, it will look like this:\n\n```elixir\ndefmodule MyApp.Blog.Post do\n  @enforce_keys [:id, :author, :title, :body, :description, :tags, :date]\n  defstruct [:id, :author, :title, :body, :description, :tags, :date]\n\n  def build(path, body, attrs) do\n    [year, month_day_id] = path |> Path.rootname() |> Path.split() |> Enum.take(-2)\n    [month, day, id] = String.split(month_day_id, \"-\", parts: 3)\n    date = Date.from_iso8601!(\"#{year}-#{month}-#{day}\")\n    struct!(__MODULE__, [id: id, date: date, body: body] ++ Map.to_list(attrs))\n  end\nend\n```\n\nNow, we are ready to define our `MyApp.Blog` with `FsBuild`:\n\n```elixir\ndefmodule MyApp.Blog do\n  alias FsBuild.Adapters.MarkdownPublisher\n  alias MyApp.Blog.Post\n\n  use FsBuild,\n    from: Application.app_dir(:my_app, \"priv/posts/**/*.md\"),\n    adapter: {MarkdownPublisher, highlighters: [:makeup_elixir, :makeup_erlang]},\n    build: Post,\n    as: :posts\n\n  # The @posts module attribute is first defined by FsBuild.\n  # Let's further modify it by sorting all posts by descending date.\n  @posts Enum.sort_by(@posts, & &1.date, {:desc, Date})\n\n  # Let's also get all tags\n  @tags @posts |> Enum.flat_map(& &1.tags) |> Enum.uniq() |> Enum.sort()\n\n  # And finally export them\n  def all_posts, do: @posts\n  def all_tags, do: @tags\nend\n```\n\n**Important**: Avoid injecting the `@posts` module attribute into multiple\nfunctions, as each call will make a complete copy of all posts. For example,\nif you want to show define `recent_posts()` as well as `all_posts()`,\n**DO NOT** do this:\n\n```elixir\ndef all_posts, do: @posts\ndef recent_posts, do: Enum.take(@posts, 3)\n```\n\nInstead do this:\n\n```elixir\ndef all_posts, do: @posts\ndef recent_posts, do: Enum.take(all_posts(), 3)\n```\n\n#","ref":"FsBuild.html#module-an-example"},{"type":"module","title":"Other helpers - FsBuild","doc":"You may want to define other helpers to traverse your published resources.\nFor example, if you want to get posts by ID or with a given tag, you can\ndefine additional functions as shown below:\n\n```elixir\ndefmodule NotFoundError do\n  defexception [:message, plug_status: 404]\nend\n\ndef get_post_by_id!(id) do\n  Enum.find(all_posts(), &(&1.id == id)) ||\n    raise NotFoundError, \"post with id=#{id} not found\"\nend\n\ndef get_posts_by_tag!(tag) do\n  case Enum.filter(all_posts(), &(tag in &1.tags)) do\n    [] -> raise NotFoundError, \"posts with tag=#{tag} not found\"\n    posts -> posts\n  end\nend\n```\n\n#","ref":"FsBuild.html#module-other-helpers"},{"type":"module","title":"Live reloading - FsBuild","doc":"If you are using Phoenix, you can enable live reloading by simply telling\nPhoenix to watch the `priv/posts/` directory. Open up `config/dev.exs`,\nsearch for `live_reload:` and add this to the list of patterns:\n\n```elixir\nlive_reload: [\n  patterns: [\n    ...,\n    ~r\"priv/posts/*/.*(md)$\"\n  ]\n]\n```","ref":"FsBuild.html#module-live-reloading"},{"type":"module","title":"Custom adapters - FsBuild","doc":"By using custom adapters, you can process any kind of files. For example,\n[org](https://orgmode.org/) files, JSON files, YAML files, etc.\n\nCheckout `FsBuild.Adapter` for more details.","ref":"FsBuild.html#module-custom-adapters"},{"type":"behaviour","title":"FsBuild.Adapter","doc":"The specification of adapters.","ref":"FsBuild.Adapter.html"},{"type":"callback","title":"FsBuild.Adapter.init/1","doc":"Prepares for following works, such as:\n\n  * checking required dependencies\n  * starting necessary applications\n  * ...","ref":"FsBuild.Adapter.html#c:init/1"},{"type":"callback","title":"FsBuild.Adapter.transform/3","doc":"Transforms the content of files.\n\nIt must return:\n\n  * a 2 element tuple with attributes and body - `{data, metadata}`\n  * a list of 2 element tuple with attributes and body - `[{data, metadata} | _]`","ref":"FsBuild.Adapter.html#c:transform/3"},{"type":"type","title":"FsBuild.Adapter.content/0","doc":"The content of file.","ref":"FsBuild.Adapter.html#t:content/0"},{"type":"type","title":"FsBuild.Adapter.data/0","doc":"The main data transformed from file.","ref":"FsBuild.Adapter.html#t:data/0"},{"type":"type","title":"FsBuild.Adapter.metadata/0","doc":"The metadata transformed from file.","ref":"FsBuild.Adapter.html#t:metadata/0"},{"type":"type","title":"FsBuild.Adapter.opts/0","doc":"The option for current adapter.","ref":"FsBuild.Adapter.html#t:opts/0"},{"type":"type","title":"FsBuild.Adapter.path/0","doc":"The path of file.","ref":"FsBuild.Adapter.html#t:path/0"},{"type":"module","title":"FsBuild.Adapters.MarkdownPublisher","doc":"An adapter for building a Markdown publishing system.","ref":"FsBuild.Adapters.MarkdownPublisher.html"},{"type":"module","title":"Required dependencies - FsBuild.Adapters.MarkdownPublisher","doc":"* `:earmark`\n  * `:makeup`","ref":"FsBuild.Adapters.MarkdownPublisher.html#module-required-dependencies"},{"type":"module","title":"Options - FsBuild.Adapters.MarkdownPublisher","doc":"* `:earmark_options` - an `%Earmark.Options{}` struct\n  * `:highlighters` - which code highlighters to use. `FsBuild`\n    uses `Makeup` for syntax highlighting and you will need to add its\n    `.css` classes. You can generate the CSS classes by calling\n    `Makeup.stylesheet(:vim_style, \"makeup\")` inside iex -S mix.\n    You can replace `:vim_style` by any style of your choice\n    [defined here](https://elixir-makeup.github.io/makeup_demo/elixir.html).","ref":"FsBuild.Adapters.MarkdownPublisher.html#module-options"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.1.1 - Changelog","doc":"#","ref":"changelog.html#v1-1-1"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix compile error when Earmark is not installed","ref":"changelog.html#fixes"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"#","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add function support for `:build` option. Following forms of functions are supported:\n  - `fn path, body, attrs -> _ end`\n  - `&Mod.fun/3`","ref":"changelog.html#enhancements"}],"content_type":"text/markdown"}